import pandas as pd


def generate_diagnostics(df: pd.DataFrame) -> pd.DataFrame:
    """
    Generate a DataFrame with diagnostics to understand how the standardization process impacts the raw data.

    This function groups the output data by the standardized plaintiff name and aggregates the raw values into a string
    to make it easier to visually inspect the output. This method also adds a count of the number of raw values that were standardized
    to the same value.

    """
    count_raw_names_by_clean_name = (
        df.groupby("first_plaintiff_clean")
        .agg(
            count_first_plaintiff=("first_plaintiff", "nunique"),
            values_first_plaintiff=("first_plaintiff", lambda x: ",".join(x)),
        )
        .reset_index()
    )

    return count_raw_names_by_clean_name



def ngrams(string, n=3):
    """take in string and create ngrams of length 3 for TF-IDF vectorization"""
    ngrams = zip(*[string[i:] for i in range(n)])
    return ["".join(ngram) for ngram in ngrams]



import pandas as pd
from loguru import logger


def merge_to_raw_df(raw_df: pd.DataFrame, mapping: dict) -> pd.DataFrame:
    """
    append the clean name to the original dataset and return the appended DataFrame.
    """

    dict_for_df = [
        {"first_plaintiff": k, "first_plaintiff_clean": v} for k, v in mapping.items()
    ]
    mapping_df = pd.DataFrame.from_dict(dict_for_df)
    merged_df = raw_df.merge(mapping_df, on=["first_plaintiff"])
    return merged_df


def map_values(aStr: str, aDict: dict) -> str:
    """
    Look up if str in dict otherwise return ""
    """

    try:
        return aDict[aStr]
    except KeyError:
        return ""


def combine_columns(clean, person, company):
    if company == "" and person == "":
        return clean

    if company != "" and person != "":
        logger.warning("NAME HAS BOTH COMPANY AND PERSON NAME: " + clean)
        return clean
    if company != "":
        return company

    if person != "":
        return person


def combine_columns_no_space(all_names, no_space):
    if no_space == "":
        return all_names

    return no_space



import probablepeople as pp
import re


def filter_companies(name):

    """Take in a particular string and determine whether or not the string is a person using the probablepeople library
    Assert that
        1) ProbablePeople doesn't tag the name as a person OR
        2) the substring `|and|` is found in the name, which is much easier to parse as a company OR
        3) The name contains no spaces, which ProbablePeople likes to parse as a person AND
        4) The name does not start with a string, which these methods are poor at parsing right now"""

    # IMPORTANT: Cosine similarity (and really any fuzzy string match) doesn't work well on people
    # Think Smith, Tom A. and Smith, Tom B. -> are these people the same? A fuzzy string match would say yes. But can we be sure?
    # As such, for this its better to remove people from the output all together and deal with them later in the normalization process.
    # Fortunately the probablepeople library can be used to quickly identify strings that are people, and just ignore them.

    # Probable people performs poorly on bank names. Working to update training dataset but this is a quick fix for now

    try:
        val = pp.tag(name)
        return (
            (val[1] != "Person")
            or 'CorporationNameOrganization' in val[0]
            or 'CorporationNameBranchIdentifier' in val[0]
            or 'CorporationLegalType' in val[0]
            or ("|and|" in name)
            or (name.count(" ") == 0)
            or (re.search(" bank$", name))
            or 'citibank' in name
            or ("bennington crossing" in name)
        ) and not name[0].isdigit()
    except:
        return True


def filter_people(name):
    """Take in a particular string and determine whether or not the string is a person using the probablepeople library
    Assert that
        1) ProbablePeople tags the name as a person AND
        2) the substring `|and|` is not found in the name, which is much easier to parse as a company AND
        3) The name contains spaces (mononyms are often read as people by probablepeople. But more frequently refer to company names)
    """

    try:
        val = pp.tag(name)
        return (
            (val[1] == "Person")
            and 'CorporationNameOrganization' not in val[0]
            and 'CorporationNameBranchIdentifier' not in val[0]
            and 'CorporationLegalType' not in val[0]
            and 'citibank' not in name
            and ("|and|" not in name)
            and (name.count(" ") != 0)
            and not (re.search(" bank$", name))
            and ("bennington crossing" not in name)
        )
    except:
        return False
